# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for the passphrase class.
"""
from castellan.common import objects
from castellan.common.objects import passphrase
from castellan.tests import base


class PassphraseTestCase(base.TestCase):

    def _create_passphrase(self):
        return passphrase.Passphrase(self.passphrase_data,
                                     self.name,
                                     self.created,
                                     consumers=self.consumers)

    def setUp(self):
        self.passphrase_data = bytes(b"secret passphrase")
        self.name = 'my phrase'
        self.created = 1448088699
        self.consumers = [{'service': 'service_test',
                           'resource_type': 'type_test',
                           'resource_id': 'id_test'}]
        self.passphrase = self._create_passphrase()

        super(PassphraseTestCase, self).setUp()

    def test_is_not_only_metadata(self):
        self.assertFalse(self.passphrase.is_metadata_only())

    def test_is_only_metadata(self):
        p = passphrase.Passphrase(None, self.name, self.created)
        self.assertTrue(p.is_metadata_only())

    def test_get_format(self):
        self.assertEqual('RAW', self.passphrase.format)

    def test_get_encoded(self):
        self.assertEqual(self.passphrase_data, self.passphrase.get_encoded())

    def test_get_name(self):
        self.assertEqual(self.name, self.passphrase.name)

    def test_get_created(self):
        self.assertEqual(self.created, self.passphrase.created)

    def test_get_consumers(self):
        self.assertEqual(self.consumers, self.passphrase.consumers)

    def test_get_created_none(self):
        created = None
        phrase = passphrase.Passphrase(self.passphrase_data,
                                       self.name,
                                       created,
                                       consumers=self.consumers)

        self.assertEqual(created, phrase.created)

    def test___eq__(self):
        self.assertTrue(self.passphrase == self.passphrase)
        self.assertTrue(self.passphrase is self.passphrase)

        self.assertFalse(self.passphrase is None)
        self.assertFalse(None == self.passphrase)  # noqa: E711

        other_passphrase = passphrase.Passphrase(self.passphrase_data)
        self.assertTrue(self.passphrase == other_passphrase)
        self.assertFalse(self.passphrase is other_passphrase)

    def test___ne___none(self):
        self.assertTrue(self.passphrase is not None)
        self.assertTrue(None != self.passphrase)  # noqa: E711

    def test___ne___data(self):
        other_phrase = passphrase.Passphrase(b"other passphrase", self.name)
        self.assertTrue(self.passphrase != other_phrase)

    def test_to_and_from_dict(self):
        other = objects.from_dict(self.passphrase.to_dict())
        self.assertEqual(self.passphrase, other)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Key manager implementation that raises NotImplementedError
"""

from castellan.key_manager import key_manager


class NotImplementedKeyManager(key_manager.KeyManager):
    """Key Manager Interface that raises NotImplementedError for all operations

    """

    def __init__(self, configuration=None):
        super(NotImplementedKeyManager, self).__init__(configuration)

    def create_key(self, context, algorithm='AES', length=256,
                   expiration=None, name=None, **kwargs):
        raise NotImplementedError()

    def create_key_pair(self, context, algorithm, length,
                        expiration=None, name=None):
        raise NotImplementedError()

    def store(self, context, managed_object, expiration=None, **kwargs):
        raise NotImplementedError()

    def copy(self, context, managed_object_id, **kwargs):
        raise NotImplementedError()

    def get(self, context, managed_object_id, **kwargs):
        raise NotImplementedError()

    def list(self, context, object_type=None):
        raise NotImplementedError()

    def delete(self, context, managed_object_id, force=False):
        raise NotImplementedError()

    def add_consumer(self, context, managed_object_id, consumer_data):
        raise NotImplementedError()

    def remove_consumer(self, context, managed_object_id, consumer_data):
        raise NotImplementedError()



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for the X.509 certificate class.
"""
from castellan.common import objects
from castellan.common.objects import x_509
from castellan.tests import base
from castellan.tests import utils


class X509TestCase(base.CertificateTestCase):

    def _create_cert(self):
        return x_509.X509(self.data,
                          self.name,
                          self.created,
                          consumers=self.consumers)

    def setUp(self):
        self.data = utils.get_certificate_der()
        self.name = 'my cert'
        self.created = 1448088699
        self.consumers = [{'service': 'service_test',
                           'resource_type': 'type_test',
                           'resource_id': 'id_test'}]

        super(X509TestCase, self).setUp()

    def test_is_not_only_metadata(self):
        self.assertFalse(self.cert.is_metadata_only())

    def test_is_only_metadata(self):
        c = x_509.X509(None, self.name, self.created)
        self.assertTrue(c.is_metadata_only())

    def test_get_format(self):
        self.assertEqual('X.509', self.cert.format)

    def test_get_name(self):
        self.assertEqual(self.name, self.cert.name)

    def test_get_encoded(self):
        self.assertEqual(self.data, self.cert.get_encoded())

    def test_get_created(self):
        self.assertEqual(self.created, self.cert.created)

    def test_get_consumers(self):
        self.assertEqual(self.consumers, self.cert.consumers)

    def test_get_created_none(self):
        created = None
        cert = x_509.X509(self.data,
                          self.name,
                          created,
                          consumers=self.consumers)

        self.assertEqual(created, cert.created)

    def test___eq__(self):
        self.assertTrue(self.cert == self.cert)
        self.assertTrue(self.cert is self.cert)

        self.assertFalse(self.cert is None)
        self.assertFalse(None == self.cert)  # noqa: E711

        other_x_509 = x_509.X509(self.data)
        self.assertTrue(self.cert == other_x_509)
        self.assertFalse(self.cert is other_x_509)

    def test___ne___none(self):
        self.assertTrue(self.cert is not None)
        self.assertTrue(None != self.cert)  # noqa: E711

    def test___ne___data(self):
        other_x509 = x_509.X509(b'\x00\x00\x00', self.name)
        self.assertTrue(self.cert != other_x509)

    def test___ne___consumers(self):
        different_consumers = [{'service': 'other_service',
                                'resource_type': 'other_type',
                                'resource_id': 'other_id'}]
        other_cert = x_509.X509(self.data,
                                self.name,
                                self.created,
                                consumers=different_consumers)
        self.assertTrue(self.cert is not other_cert)

    def test_to_and_from_dict(self):
        other = objects.from_dict(self.cert.to_dict())
        self.assertEqual(self.cert, other)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Castellan exception subclasses
"""

import urllib.parse

from castellan.i18n import _

_FATAL_EXCEPTION_FORMAT_ERRORS = False


class RedirectException(Exception):
    def __init__(self, url):
        self.url = urllib.parse.urlparse(url)


class CastellanException(Exception):
    """Base Castellan Exception

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.
    """
    message = _("An unknown exception occurred")

    def __init__(self, message_arg=None, *args, **kwargs):
        if not message_arg:
            message_arg = self.message
        try:
            self.message = message_arg % kwargs
        except Exception:
            if _FATAL_EXCEPTION_FORMAT_ERRORS:
                raise
            else:
                # at least get the core message out if something happened
                pass
        super(CastellanException, self).__init__(self.message)


class Forbidden(CastellanException):
    message = _("You are not authorized to complete this action.")


class KeyManagerError(CastellanException):
    message = _("Key manager error: %(reason)s")


class ManagedObjectNotFoundError(CastellanException):
    message = _("Key not found, uuid: %(uuid)s")


class InvalidManagedObjectDictError(CastellanException):
    message = _("Dict has no field '%(field)s'.")


class UnknownManagedObjectTypeError(CastellanException):
    message = _("Type not found, type: %(type)s")


class AuthTypeInvalidError(CastellanException):
    message = _("Invalid auth_type was specified, auth_type: %(type)s")


class InsufficientCredentialDataError(CastellanException):
    message = _("Insufficient credential data was provided, either "
                "\"token\" must be set in the passed conf, or a context "
                "with an \"auth_token\" property must be passed.")



# Copyright (c) 2015 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import warnings

from oslo_config import cfg
from oslo_log import log
from stevedore import ExtensionManager

from castellan import key_manager
from castellan.key_manager import barbican_key_manager as bkm
from castellan.key_manager import vault_key_manager as vkm

from castellan.common import utils

_DEFAULT_LOG_LEVELS = ['castellan=WARN']


def set_defaults(conf, backend=None, barbican_endpoint=None,
                 barbican_api_version=None, auth_endpoint=None,
                 retry_delay=None, number_of_retries=None, verify_ssl=None,
                 verify_ssl_path=None,
                 api_class=None, vault_root_token_id=None,
                 vault_approle_role_id=None, vault_approle_secret_id=None,
                 vault_kv_mountpoint=None, vault_kv_path=None, vault_url=None,
                 vault_ssl_ca_crt_file=None, vault_use_ssl=None,
                 vault_namespace=None,
                 barbican_endpoint_type=None,
                 vault_kv_version=None):
    """Set defaults for configuration values.

    Overrides the default options values.
    :param conf: Config instance in which to set default options.
    :param api_class: The full class name of the key manager API class.
    :param barbican_endpoint: Use this endpoint to connect to Barbican.
    :param barbican_api_version: Version of the Barbican API.
    :param auth_endpoint: Use this endpoint to connect to Keystone.
    :param retry_delay: Use this attribute to set retry delay.
    :param number_of_retries: Use this attribute to set number of retries.
    :param verify_ssl: Use this to specify if ssl should be verified.
    :param verify_ssl_path: Use this to specify the CA path.
    :param vault_root_token_id: Use this for the root token id for vault.
    :param vault_approle_role_id: Use this for the approle role_id for vault.
    :param vault_approle_secret_id: Use this for the approle secret_id
                                    for vault.
    :param vault_kv_mountpoint: Mountpoint of KV store in vault to use.
    :param vault_kv_path: Path relative to root of KV store in Vault to use.
    :param vault_url: Use this for the url for vault.
    :param vault_use_ssl: Use this to force vault driver to use ssl.
    :param vault_ssl_ca_crt_file: Use this for the CA file for vault.
    :param vault_namespace: Namespace to use for all requests to Vault.
    :param barbican_endpoint_type: Use this to specify the type of URL.
    :                              Valid values are: public, internal or admin.
    :param vault_kv_version: Use this for the kv version for vault.
    """
    conf.register_opts(key_manager.key_manager_opts, group='key_manager')

    ext_mgr = ExtensionManager(
        "castellan.drivers",
        invoke_on_load=True,
        invoke_args=[cfg.CONF])

    for km in ext_mgr.names():
        for group, opts in ext_mgr[km].obj.list_options_for_discovery():
            conf.register_opts(opts, group=group)

    # Use the new backend option if set or fall back to the older api_class
    default_backend = backend or api_class
    if default_backend is not None:
        conf.set_default('backend', default_backend, group='key_manager')

    if barbican_endpoint is not None:
        conf.set_default('barbican_endpoint', barbican_endpoint,
                         group=bkm._BARBICAN_OPT_GROUP)
    if barbican_api_version is not None:
        conf.set_default('barbican_api_version', barbican_api_version,
                         group=bkm._BARBICAN_OPT_GROUP)
    if auth_endpoint is not None:
        conf.set_default('auth_endpoint', auth_endpoint,
                         group=bkm._BARBICAN_OPT_GROUP)
    if retry_delay is not None:
        conf.set_default('retry_delay', retry_delay,
                         group=bkm._BARBICAN_OPT_GROUP)
    if number_of_retries is not None:
        conf.set_default('number_of_retries', number_of_retries,
                         group=bkm._BARBICAN_OPT_GROUP)
    if verify_ssl is not None:
        conf.set_default('verify_ssl', verify_ssl,
                         group=bkm._BARBICAN_OPT_GROUP)
    if verify_ssl_path is not None:
        conf.set_default('verify_ssl_path', verify_ssl_path,
                         group=bkm._BARBICAN_OPT_GROUP)
    if barbican_endpoint_type is not None:
        conf.set_default('barbican_endpoint_type', barbican_endpoint_type,
                         group=bkm._BARBICAN_OPT_GROUP)

    if vault_root_token_id is not None:
        conf.set_default('root_token_id', vault_root_token_id,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_approle_role_id is not None:
        conf.set_default('approle_role_id', vault_approle_role_id,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_approle_secret_id is not None:
        conf.set_default('approle_secret_id', vault_approle_secret_id,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_kv_mountpoint is not None:
        conf.set_default('kv_mountpoint', vault_kv_mountpoint,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_kv_path is not None:
        conf.set_default('kv_path', vault_kv_path,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_url is not None:
        conf.set_default('vault_url', vault_url,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_ssl_ca_crt_file is not None:
        conf.set_default('ssl_ca_crt_file', vault_ssl_ca_crt_file,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_use_ssl is not None:
        warnings.warn('use_ssl option is deprecated',
                      category=DeprecationWarning)
    if vault_namespace is not None:
        conf.set_default('namespace', vault_namespace,
                         group=vkm._VAULT_OPT_GROUP)
    if vault_kv_version is not None:
        conf.set_default('kv_version', vault_kv_version,
                         group=vkm._VAULT_OPT_GROUP)


def enable_logging(conf=None, app_name='castellan'):
    conf = conf or cfg.CONF

    log.register_options(conf)
    log.set_defaults(default_log_levels=_DEFAULT_LOG_LEVELS)

    log.setup(conf, app_name)


def list_opts():
    """Returns a list of oslo.config options available in the library.

    The returned list includes all oslo.config options which may be registered
    at runtime by the library.

    Each element of the list is a tuple. The first element is the name of the
    group under which the list of elements in the second element will be
    registered. A group name of None corresponds to the [DEFAULT] group in
    config files.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users by this library.

    :returns: a list of (group_name, opts) tuples
    """
    key_manager_opts = []
    key_manager_opts.extend(key_manager.key_manager_opts)
    key_manager_opts.extend(utils.credential_opts)
    opts = [('key_manager', key_manager_opts)]

    ext_mgr = ExtensionManager(
        "castellan.drivers",
        invoke_on_load=True,
        invoke_args=[cfg.CONF])

    for driver in ext_mgr.names():
        opts.extend(ext_mgr[driver].obj.list_options_for_discovery())

    return opts



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Functional test cases for the Barbican key manager.

Note: This requires local running instances of Barbican and Keystone.
"""

import abc
import unittest

from keystoneauth1 import identity
from keystoneauth1 import session
from oslo_config import cfg
from oslo_context import context
from oslo_utils import uuidutils
from oslotest import base

from castellan.common.credentials import keystone_password
from castellan.common.credentials import keystone_token
from castellan.common import exception
from castellan.key_manager import barbican_key_manager
from castellan.tests.functional import config
from castellan.tests.functional.key_manager import test_key_manager
from castellan.tests import utils


CONF = config.get_config()


@utils.parameterized_test_case
class BarbicanKeyManagerTestCase(test_key_manager.KeyManagerTestCase):

    def _create_key_manager(self):
        return barbican_key_manager.BarbicanKeyManager(cfg.CONF)

    @abc.abstractmethod
    def get_context(self):
        """Retrieves Context for Authentication"""
        return

    def setUp(self):
        super(BarbicanKeyManagerTestCase, self).setUp()
        try:
            self.ctxt = self.get_context()
            self.key_mgr._get_barbican_client(self.ctxt)
        except Exception as e:
            # When we run functional-vault target, This test class needs
            # to be skipped as barbican is not running
            raise unittest.SkipTest(str(e))

    def tearDown(self):
        super(BarbicanKeyManagerTestCase, self).tearDown()

    def test_create_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.create_key, None, 'AES', 256)

    def test_create_key_pair_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.create_key_pair, None, 'RSA', 2048)

    def test_delete_null_context(self):
        key_uuid = self._get_valid_object_uuid(
            test_key_manager._get_test_symmetric_key())
        self.addCleanup(self.key_mgr.delete, self.ctxt, key_uuid)
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.delete, None, key_uuid)

    def test_delete_null_object(self):
        self.assertRaises(exception.KeyManagerError,
                          self.key_mgr.delete, self.ctxt, None)

    def test_delete_unknown_object(self):
        unknown_uuid = uuidutils.generate_uuid()
        self.assertRaises(exception.ManagedObjectNotFoundError,
                          self.key_mgr.delete, self.ctxt, unknown_uuid)

    def test_get_null_context(self):
        key_uuid = self._get_valid_object_uuid(
            test_key_manager._get_test_symmetric_key())
        self.addCleanup(self.key_mgr.delete, self.ctxt, key_uuid)
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.get, None, key_uuid)

    def test_get_null_object(self):
        self.assertRaises(exception.KeyManagerError,
                          self.key_mgr.get, self.ctxt, None)

    def test_get_unknown_key(self):
        bad_key_uuid = uuidutils.generate_uuid()
        self.assertRaises(exception.ManagedObjectNotFoundError,
                          self.key_mgr.get, self.ctxt, bad_key_uuid)

    def test_store_null_context(self):
        key = test_key_manager._get_test_symmetric_key()

        self.assertRaises(exception.Forbidden,
                          self.key_mgr.store, None, key)

    def test_secret_create_check_empty_consumers_list(self):
        """Check that the consumers entity is a list and is empty."""

        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        resp = self.key_mgr.get(self.ctxt, stored_id)
        consumers = resp.consumers
        self.assertIsInstance(consumers, list)
        self.assertEqual(len(consumers), 0)

    def test_secret_create_check_consumers_list_consistency(self):
        """Consumers List Consistency

        Check that the consumers list contains a single element,
        and that it corresponds to the consumer created.
        """

        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        resource_id = uuidutils.generate_uuid()
        consumer_data = {
            'service': 'dummy_service',
            'resource_type': 'dummy_resource_type',
            'resource_id': resource_id
        }
        self.key_mgr.add_consumer(self.ctxt, stored_id, consumer_data)
        stored_secret = self.key_mgr.get(self.ctxt, stored_id)
        self.assertIsNotNone(stored_secret)
        self.assertIsInstance(stored_secret.consumers, list)
        self.assertEqual(len(stored_secret.consumers), 1)
        self.assertEqual(stored_secret.consumers[0]['service'],
                         consumer_data['service'])
        self.assertEqual(stored_secret.consumers[0]['resource_type'],
                         consumer_data['resource_type'])
        self.assertEqual(stored_secret.consumers[0]['resource_id'],
                         consumer_data['resource_id'])

    def test_secret_create_remove_nonexistent_consumer(self):
        """Removing a nonexistent consumer should raise an exception."""
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        resource_id = uuidutils.generate_uuid()
        consumer_data = {
            'service': 'dummy_service',
            'resource_type': 'dummy_resource_type',
            'resource_id': resource_id
        }
        self.assertRaises(exception.ManagedObjectNotFoundError,
                          self.key_mgr.remove_consumer, self.ctxt,
                          stored_id, consumer_data)

    @utils.parameterized_dataset({
        'remove_one': [[{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}],
                       [{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'}]],
        'remove_all': [[{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}],
                       [{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}]]
    })
    def test_secret_create_and_adding_removing_consumers(
            self,
            add_consumers,
            remove_consumers):
        """The following activities are carried:

        Create a secret, then register each consumer
        in the register_consumers list, then remove each consumer
        in the remove_consumers list.
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        for consumer in add_consumers:
            self.key_mgr.add_consumer(self.ctxt, stored_id, consumer)
        stored_secret = self.key_mgr.get(self.ctxt, stored_id)
        self.assertCountEqual(add_consumers, stored_secret.consumers)

        for consumer in remove_consumers:
            self.key_mgr.remove_consumer(self.ctxt, stored_id, consumer)
        stored_secret = self.key_mgr.get(self.ctxt, stored_id)

        removed_ids = set([v['resource_id'] for v in remove_consumers])
        remaining_consumers = [v for v in add_consumers
                               if v['resource_id'] not in removed_ids]
        self.assertCountEqual(remaining_consumers, stored_secret.consumers)

    @utils.parameterized_dataset({
        'no_args': [[{}]],
        'one_arg_1': [[{'service': 'service1'}]],
        'one_arg_2': [[{'resource_type': 'type1'}]],
        'one_arg_3': [[{'resource_id': 'id1'}]],
        'two_args_1': [[{'service': 'service1',
                         'resource_type': 'type1'}]],
        'two_args_2': [[{'service': 'service1',
                         'resource_id': 'id1'}]],
        'two_args_3': [[{'resource_type': 'type1',
                         'resource_id': 'id'}]]
    })
    def test_consumer_add_missing_positional_arguments(self, consumers):
        """Missing Positional Arguments - Addition

        Tries to add a secret consumer without providing all of the required
        positional arguments (service, resource_type, resource_id).
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        for consumer in consumers:
            e = self.assertRaises(
                TypeError,
                self.key_mgr.add_consumer,
                self.ctxt, stored_id, consumer)
        self.assertIn('register_consumer() missing', str(e))

    @utils.parameterized_dataset({
        'no_args': [[{}]],
        'one_arg_1': [[{'service': 'service1'}]],
        'one_arg_2': [[{'resource_type': 'type1'}]],
        'one_arg_3': [[{'resource_id': 'id1'}]],
        'two_args_1': [[{'service': 'service1',
                         'resource_type': 'type1'}]],
        'two_args_2': [[{'service': 'service1',
                         'resource_id': 'id1'}]],
        'two_args_3': [[{'resource_type': 'type1',
                         'resource_id': 'id'}]]
    })
    def test_consumer_remove_missing_positional_arguments(self, consumers):
        """Missing Positional Arguments - Removal

        Tries to remove a secret consumer without providing all of the required
        positional arguments (service, resource_type, resource_id).
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        consumer_data = {
            'service': 'service1',
            'resource_type': 'type1',
            'resource_id': 'id1'
        }
        self.key_mgr.add_consumer(self.ctxt, stored_id, consumer_data)
        for consumer in consumers:
            e = self.assertRaises(
                TypeError,
                self.key_mgr.remove_consumer,
                self.ctxt, stored_id, consumer)
        self.assertIn('remove_consumer() missing', str(e))

    def test_consumer_add_two_remove_one_check_consumers_list(self):
        """Consumers addition and removal - check of list consistency

        Adds two consumers, removes one and verifies if the consumers
        list's length is consistent (equals to 1).
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        consumers = [
            {'service': 'service1',
             'resource_type': 'type1',
             'resource_id': 'id1'},
            {'service': 'service2',
             'resource_type': 'type2',
             'resource_id': 'id2'}
        ]
        for consumer in consumers:
            self.key_mgr.add_consumer(self.ctxt, stored_id, consumer)
        stored_secret = self.key_mgr.get(self.ctxt, stored_id)
        self.assertCountEqual(consumers, stored_secret.consumers)

        self.key_mgr.remove_consumer(self.ctxt, stored_id, consumers[0])
        stored_secret = self.key_mgr.get(self.ctxt, stored_id)
        self.assertCountEqual(consumers[1:], stored_secret.consumers)

    def test_consumer_add_secret_delete_force_parameter_nonexisting(self):
        """Consumer addition, secret deletion with no 'force' parameter

        Creates a secret, adds a consumer to it and tries to delete the secret
        without specifying the 'force' parameter.
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        consumer = {'service': 'service1',
                    'resource_type': 'type1',
                    'resource_id': 'id1'}
        self.key_mgr.add_consumer(self.ctxt, stored_id, consumer)

        e = self.assertRaises(ValueError, self.key_mgr.delete,
                              self.ctxt, stored_id)
        self.assertIn("Secret has consumers! Remove them first or use the "
                      "force parameter to delete it.", str(e))

    def test_consumer_add_secret_delete_force_parameter_false(self):
        """Consumer addition, secret deletion, 'force' parameter equals False

        Creates a secret, adds a consumer to it and tries to delete the secret
        specifying the 'force' parameter as False.
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.addCleanup(self.key_mgr.delete, self.ctxt, stored_id, True)
        self.assertIsNotNone(stored_id)

        consumer = {'service': 'service1',
                    'resource_type': 'type1',
                    'resource_id': 'id1'}
        self.key_mgr.add_consumer(self.ctxt, stored_id, consumer)

        e = self.assertRaises(ValueError, self.key_mgr.delete,
                              self.ctxt, stored_id, False)
        self.assertIn("Secret has consumers! Remove them first or use the "
                      "force parameter to delete it.", str(e))

    def test_consumer_add_secret_delete_force_parameter_true(self):
        """Consumer addition, secret deletion, 'force' parameter equals True

        Creates a secret, adds a consumer to it and deletes the secret,
        specifying the 'force' parameter as True.
        """
        key = test_key_manager._get_test_passphrase()
        self.assertIsNotNone(key)

        stored_id = self.key_mgr.store(self.ctxt, key)
        self.assertIsNotNone(stored_id)

        consumer = {'service': 'service1',
                    'resource_type': 'type1',
                    'resource_id': 'id1'}
        self.key_mgr.add_consumer(self.ctxt, stored_id, consumer)

        self.key_mgr.delete(self.ctxt, stored_id, True)


class BarbicanKeyManagerOSLOContextTestCase(BarbicanKeyManagerTestCase,
                                            base.BaseTestCase):

    def get_context(self):
        username = CONF.identity.username
        password = CONF.identity.password
        project_name = CONF.identity.project_name
        auth_url = CONF.identity.auth_url
        user_domain_name = CONF.identity.user_domain_name
        project_domain_name = CONF.identity.project_domain_name

        auth = identity.V3Password(auth_url=auth_url,
                                   username=username,
                                   password=password,
                                   project_name=project_name,
                                   user_domain_name=user_domain_name,
                                   project_domain_name=project_domain_name)
        sess = session.Session(auth=auth)

        return context.RequestContext(auth_token=auth.get_token(sess),
                                      tenant=auth.get_project_id(sess))


class BarbicanKeyManagerKSPasswordTestCase(BarbicanKeyManagerTestCase,
                                           base.BaseTestCase):

    def get_context(self):
        auth_url = CONF.identity.auth_url
        username = CONF.identity.username
        password = CONF.identity.password
        project_name = CONF.identity.project_name
        user_domain_name = CONF.identity.user_domain_name
        project_domain_name = CONF.identity.project_domain_name

        ctxt = keystone_password.KeystonePassword(
            auth_url=auth_url, username=username, password=password,
            project_name=project_name,
            user_domain_name=user_domain_name,
            project_domain_name=project_domain_name)

        return ctxt


class BarbicanKeyManagerKSTokenTestCase(BarbicanKeyManagerTestCase,
                                        base.BaseTestCase):

    def get_context(self):
        username = CONF.identity.username
        password = CONF.identity.password
        project_name = CONF.identity.project_name
        auth_url = CONF.identity.auth_url
        user_domain_name = CONF.identity.user_domain_name
        project_domain_name = CONF.identity.project_domain_name

        auth = identity.V3Password(auth_url=auth_url,
                                   username=username,
                                   password=password,
                                   project_name=project_name,
                                   user_domain_name=user_domain_name,
                                   project_domain_name=project_domain_name)
        sess = session.Session()

        return keystone_token.KeystoneToken(
            token=auth.get_token(sess),
            auth_url=auth_url,
            project_id=auth.get_project_id(sess))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Functional test cases for the Castellan Oslo Config Driver.

Note: This requires local running instance of Vault.
"""
import tempfile

from oslo_config import cfg
from oslo_config import fixture

from oslotest import base

from castellan import _config_driver
from castellan.common.objects import opaque_data
from castellan.tests.unit.key_manager import fake


class CastellanSourceTestCase(base.BaseTestCase):

    def setUp(self):
        super(CastellanSourceTestCase, self).setUp()
        self.driver = _config_driver.CastellanConfigurationSourceDriver()
        self.conf = cfg.ConfigOpts()
        self.conf_fixture = self.useFixture(fixture.Config(self.conf))

    def test_incomplete_driver(self):
        # The group exists, but does not specify the
        # required options for this driver.
        self.conf_fixture.load_raw_values(
            group='incomplete_driver',
            driver='castellan',
        )
        source = self.conf._open_source_from_opt_group('incomplete_driver')

        self.assertIsNone(source)
        self.assertEqual(self.conf.incomplete_driver.driver, 'castellan')

    def test_complete_driver(self):
        self.conf_fixture.load_raw_values(
            group='castellan_source',
            driver='castellan',
            config_file='config.conf',
            mapping_file='mapping.conf',
        )

        with base.mock.patch.object(
                _config_driver,
                'CastellanConfigurationSource') as source_class:
            self.driver.open_source_from_opt_group(
                self.conf, 'castellan_source')

            source_class.assert_called_once_with(
                'castellan_source',
                self.conf.castellan_source.config_file,
                self.conf.castellan_source.mapping_file)

    def test_fetch_secret(self):
        # fake KeyManager populated with secret
        km = fake.fake_api()
        secret_id = km.store("fake_context",
                             opaque_data.OpaqueData(b"super_secret!"))

        # driver config
        config = "[key_manager]\nbackend=vault\nauth_type=keystone_password\n"
        mapping = "[DEFAULT]\nmy_secret=" + secret_id

        # creating temp files
        with tempfile.NamedTemporaryFile() as config_file:
            config_file.write(config.encode("utf-8"))
            config_file.flush()

            with tempfile.NamedTemporaryFile() as mapping_file:
                mapping_file.write(mapping.encode("utf-8"))
                mapping_file.flush()

                self.conf_fixture.load_raw_values(
                    group='castellan_source',
                    driver='castellan',
                    config_file=config_file.name,
                    mapping_file=mapping_file.name,
                )

                source = self.driver.open_source_from_opt_group(
                    self.conf,
                    'castellan_source')

                # replacing key_manager with fake one
                source._mngr = km

                # testing if the source is able to retrieve
                # the secret value stored in the key_manager
                # using the secret_id from the mapping file
                self.assertEqual("super_secret!",
                                 source.get("DEFAULT",
                                            "my_secret",
                                            cfg.StrOpt(""))[0])

    def test_missing_auth_type(self):
        # driver config without auth_type
        config = "[key_manager]\nbackend=vault\n"
        mapping = "[DEFAULT]\nmy_secret=some_id"

        # creating temp files
        with tempfile.NamedTemporaryFile() as config_file:
            config_file.write(config.encode("utf-8"))
            config_file.flush()

            with tempfile.NamedTemporaryFile() as mapping_file:
                mapping_file.write(mapping.encode("utf-8"))
                mapping_file.flush()

                self.conf_fixture.load_raw_values(
                    group='castellan_source',
                    driver='castellan',
                    config_file=config_file.name,
                    mapping_file=mapping_file.name,
                )

                source = self.driver.open_source_from_opt_group(
                    self.conf,
                    'castellan_source')

                # testing that context is None
                self.assertIsNone(source._context)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from castellan.common import exception
from castellan.common.objects import opaque_data
from castellan.common.objects import passphrase
from castellan.common.objects import private_key
from castellan.common.objects import public_key
from castellan.common.objects import symmetric_key
from castellan.common.objects import x_509

_managed_objects_by_type = {
    cls.managed_type(): cls for cls in [
        opaque_data.OpaqueData,
        passphrase.Passphrase,
        private_key.PrivateKey,
        public_key.PublicKey,
        symmetric_key.SymmetricKey,
        x_509.X509,
    ]
}


def from_dict(obj, id=None):
    try:
        managed_object_type = obj["type"]
    except KeyError:
        raise exception.InvalidManagedObjectDictError(field="type")

    try:
        cls = _managed_objects_by_type[managed_object_type]
    except KeyError:
        raise exception.UnknownManagedObjectTypeError(type=managed_object_type)

    try:
        managed_object = cls.from_dict(obj, id)
    except KeyError as e:
        raise exception.InvalidManagedObjectDictError(field=str(e))

    return managed_object



# Copyright (c) 2015 IBM
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Base Password Credential

This module defines the Password credential.
"""

from castellan.common.credentials import credential


class Password(credential.Credential):
    """This class represents a password credential."""

    def __init__(self, username, password):
        """Create a new Password credential.

        :param string password: Password for authentication.
        :param string username: Username for authentication.
        """

        self._username = username
        self._password = password

    @property
    def username(self):
        """This method returns a username."""
        return self._username

    @property
    def password(self):
        """This method returns a password."""
        return self._password

    def __eq__(self, other):
        if isinstance(other, Password):
            return (self._username == other._username and
                    self._password == other._password)
        else:
            return False

    def __ne__(self, other):
        result = self.__eq__(other)
        return not result



# Copyright (c) 2015
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""These utility functions are borrowed from Barbican's testing utilities."""

import functools
import types


def construct_new_test_function(original_func, name, build_params):
    """Builds a new test function based on parameterized data.

    :param original_func: The original test function that is used as a template
    :param name: The fullname of the new test function
    :param build_params: A dictionary or list containing args or kwargs
        for the new test
    :return: A new function object
    """
    new_func = types.FunctionType(
        original_func.__code__,
        original_func.__globals__,
        name=name,
        argdefs=original_func.__defaults__,
    )

    for key, val in original_func.__dict__.items():
        if key != 'build_data':
            new_func.__dict__[key] = val

    # Support either an arg list or kwarg dict for our data
    build_args = build_params if isinstance(build_params, list) else []
    build_kwargs = build_params if isinstance(build_params, dict) else {}

    # Build a test wrapper to execute with our kwargs
    def test_wrapper(func, test_args, test_kwargs):
        @functools.wraps(func)
        def wrapper(self):
            return func(self, *test_args, **test_kwargs)
        return wrapper

    return test_wrapper(new_func, build_args, build_kwargs)


def process_parameterized_function(name, func_obj, build_data):
    """Build lists of functions to add and remove to a test case."""
    to_remove = []
    to_add = []

    for subtest_name, params in build_data.items():
        # Build new test function
        func_name = '{0}_{1}'.format(name, subtest_name)
        new_func = construct_new_test_function(func_obj, func_name, params)

        # Mark the new function as needed to be added to the class
        to_add.append((func_name, new_func))

        # Mark key for removal
        to_remove.append(name)

    return to_remove, to_add


def parameterized_test_case(cls):
    """Class decorator to process parameterized tests

    This allows for parameterization to be used for potentially any
    unittest compatible runner; including testr and py.test.
    """
    tests_to_remove = []
    tests_to_add = []
    for key, val in vars(cls).items():
        # Only process tests with build data on them
        if key.startswith('test_') and val.__dict__.get('build_data'):
            to_remove, to_add = process_parameterized_function(
                name=key,
                func_obj=val,
                build_data=val.__dict__.get('build_data')
            )
            tests_to_remove.extend(to_remove)
            tests_to_add.extend(to_add)

    # Add all new test functions
    [setattr(cls, name, func) for name, func in tests_to_add]

    # Remove all old test function templates (if they still exist)
    [delattr(cls, key) for key in tests_to_remove if hasattr(cls, key)]
    return cls


def parameterized_dataset(build_data):
    """Simple decorator to mark a test method for processing."""
    def decorator(func):
        func.__dict__['build_data'] = build_data
        return func
    return decorator


def get_certificate_der():
    """Returns an X509 certificate in DER format

    This certificate was created by issuing the following openssl commands:
    openssl genrsa -out private.pem 2048
    openssl req -new -x509 -key private.pem -out cert.pem \
                -days 1000 -subj '/CN=example.com'
    openssl x509 -outform der -in cert.pem -out cert.der
    The byte string returned by this function is the contents
    of the cert.der file.
    """
    cert_der = (
        b'\x30\x82\x02\xff\x30\x82\x01\xe7\xa0\x03\x02\x01\x02\x02\x09'
        b'\x00\xe2\xea\x5c\xa2\x7d\xab\xdf\xe7\x30\x0d\x06\x09\x2a\x86'
        b'\x48\x86\xf7\x0d\x01\x01\x0b\x05\x00\x30\x16\x31\x14\x30\x12'
        b'\x06\x03\x55\x04\x03\x0c\x0b\x65\x78\x61\x6d\x70\x6c\x65\x2e'
        b'\x63\x6f\x6d\x30\x1e\x17\x0d\x31\x35\x30\x34\x31\x31\x30\x32'
        b'\x31\x35\x32\x39\x5a\x17\x0d\x31\x38\x30\x31\x30\x35\x30\x32'
        b'\x31\x35\x32\x39\x5a\x30\x16\x31\x14\x30\x12\x06\x03\x55\x04'
        b'\x03\x0c\x0b\x65\x78\x61\x6d\x70\x6c\x65\x2e\x63\x6f\x6d\x30'
        b'\x82\x01\x22\x30\x0d\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01'
        b'\x01\x05\x00\x03\x82\x01\x0f\x00\x30\x82\x01\x0a\x02\x82\x01'
        b'\x01\x00\xb3\x6b\x65\x68\x0d\x79\x81\x50\xc9\xb0\x8c\x5b\xbd'
        b'\x17\xa3\x0c\xe6\xaf\xc0\x67\x55\xa3\x9d\x60\x36\x60\xd7\x4d'
        b'\xcb\x6d\xfb\x4e\xb1\x8d\xfe\x7a\x1b\x0c\x3b\xfc\x14\x10\x69'
        b'\x50\xf9\x87\x35\x9d\x38\x1f\x52\xf2\xc4\x57\x0f\xf1\x17\x85'
        b'\xad\xc2\x17\xa6\x27\xec\x45\xeb\xb6\x94\x05\x9a\xa9\x13\xf1'
        b'\xa2\xfb\xb9\x0a\xe0\x21\x7d\xe7\x0a\xbf\xe4\x61\x8c\xb5\x4b'
        b'\x27\x42\x3e\x31\x92\x1b\xef\x64\x4e\x2a\x97\xd9\x4e\x66\xfb'
        b'\x76\x19\x45\x80\x60\xf7\xbe\x40\xb9\xd4\x10\x9f\x84\x65\x56'
        b'\xdf\x9c\x39\xd8\xe6\x3f\xdb\x7c\x79\x31\xe3\xb8\xca\xfc\x79'
        b'\x9b\x23\xdc\x72\x7c\x4c\x55\x0e\x36\x2a\xe0\xeb\xcc\xaa\xa3'
        b'\x06\x54\xa3\x98\x19\xdc\xa4\x66\x31\xd0\x98\x02\x4f\xeb\x32'
        b'\x16\x61\xec\x97\xca\xce\x92\xa0\x8f\x3c\x52\xe8\xdb\x86\x10'
        b'\x9f\xee\x3f\xa6\xbd\x40\x63\x06\x99\x01\xb3\x13\x97\xdc\xe8'
        b'\x2e\xd1\x10\x8f\xab\x31\x49\xcb\x87\x71\x2f\x5e\xf2\x78\xa9'
        b'\xb4\x3c\x65\xb1\xb2\xd0\x82\xa1\x95\x68\x67\x44\xd7\x5e\xec'
        b'\xb4\x2f\x79\x40\x7e\xd4\xbc\x84\xdb\xb9\x8c\xdd\x8d\x9c\x01'
        b'\x15\xcd\x52\x83\x3f\x06\x67\xfd\xa1\x2d\x2b\x07\xba\x32\x62'
        b'\x21\x07\x2f\x02\x03\x01\x00\x01\xa3\x50\x30\x4e\x30\x1d\x06'
        b'\x03\x55\x1d\x0e\x04\x16\x04\x14\x94\xab\x60\x34\x6f\x65\xe8'
        b'\xfa\xc2\xaf\x98\xa8\x0d\xf1\x6a\xbc\x97\xa8\xfc\xda\x30\x1f'
        b'\x06\x03\x55\x1d\x23\x04\x18\x30\x16\x80\x14\x94\xab\x60\x34'
        b'\x6f\x65\xe8\xfa\xc2\xaf\x98\xa8\x0d\xf1\x6a\xbc\x97\xa8\xfc'
        b'\xda\x30\x0c\x06\x03\x55\x1d\x13\x04\x05\x30\x03\x01\x01\xff'
        b'\x30\x0d\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01\x01\x0b\x05\x00'
        b'\x03\x82\x01\x01\x00\x63\x8a\xea\xa1\x97\x33\x55\x39\x52\xeb'
        b'\x1c\x34\x32\x1a\xbd\x1f\x4c\x00\x85\x25\xd0\xd1\x12\x7b\xa1'
        b'\x66\x9e\x1d\xf7\x5f\xbe\x0e\x63\x02\x4f\xe6\xdc\x4c\x6d\x3e'
        b'\x18\x2a\x77\xad\xf1\x4e\xb8\x45\xa9\x24\xb2\xcb\x3d\xd4\x8e'
        b'\x9c\x8b\x27\x89\xbb\x0e\xb3\x22\x8f\x5e\xe0\x41\x5f\x99\x26'
        b'\x75\x82\x28\x8d\xb7\x63\x51\x34\xb0\x9e\x17\x31\xf4\x94\xc0'
        b'\x7c\xa4\xa6\xc5\x75\x92\x0b\x4a\xe7\x28\x27\x9f\x01\xfe\x38'
        b'\x32\x6e\x9f\xaa\xfa\x13\xc9\x36\xde\x19\x24\x0f\xea\x71\xf3'
        b'\x73\xb7\x8b\x68\xaf\xde\x7d\xca\xcc\xbd\x87\x5c\xb7\xe4\xde'
        b'\x4e\x41\xe3\xa9\x1f\x0b\xbb\x8a\x63\x66\xf4\x5d\x51\x06\x9d'
        b'\x40\x78\x43\xc8\xdf\x8e\x34\xa7\x4a\x0f\xd4\xeb\x8e\xf7\xcf'
        b'\x8a\x6d\x1b\xec\x0a\xbc\xf3\x93\xe3\x48\xde\x90\xa3\x86\x7d'
        b'\x1d\x74\x7a\xfa\x72\xbe\x6d\x3c\xfd\x1f\x25\x00\x4c\xc7\xc3'
        b'\x18\xd4\x2d\xd0\xbd\xef\xc9\xf5\x71\x6c\xd3\xb1\x90\x20\x5c'
        b'\x60\x8e\x21\x16\xd1\x9f\x90\xec\xdd\xe8\x1e\xeb\xda\xc6\x35'
        b'\xc0\x62\x9d\x4c\xb1\xe4\xb9\x3e\x26\xe3\xff\x40\xfd\x23\xb3'
        b'\xbe\x71\xfe\x7a\x99\xc9\xa8\x84\xbd\x8f\x0f\xb5\x89\x18\xfc'
        b'\xc5\xc0\xc0\xe8\xf3\x53')
    return cert_der


def get_private_key_der():
    """Returns a private key in DER format

    This key was created by issuing the following openssl commands:
    openssl genrsa -out private.pem 2048
    openssl pkcs8 -in private.pem -topk8 -nocrypt \
                  -outform DER -out private_pk8.der
    The byte string returned by this function is the contents
    of the private_pk8.der file.
    """
    key_der = (
        b'\x30\x82\x04\xbf\x02\x01\x00\x30\x0d\x06\x09\x2a\x86\x48\x86'
        b'\xf7\x0d\x01\x01\x01\x05\x00\x04\x82\x04\xa9\x30\x82\x04\xa5'
        b'\x02\x01\x00\x02\x82\x01\x01\x00\xb3\x6b\x65\x68\x0d\x79\x81'
        b'\x50\xc9\xb0\x8c\x5b\xbd\x17\xa3\x0c\xe6\xaf\xc0\x67\x55\xa3'
        b'\x9d\x60\x36\x60\xd7\x4d\xcb\x6d\xfb\x4e\xb1\x8d\xfe\x7a\x1b'
        b'\x0c\x3b\xfc\x14\x10\x69\x50\xf9\x87\x35\x9d\x38\x1f\x52\xf2'
        b'\xc4\x57\x0f\xf1\x17\x85\xad\xc2\x17\xa6\x27\xec\x45\xeb\xb6'
        b'\x94\x05\x9a\xa9\x13\xf1\xa2\xfb\xb9\x0a\xe0\x21\x7d\xe7\x0a'
        b'\xbf\xe4\x61\x8c\xb5\x4b\x27\x42\x3e\x31\x92\x1b\xef\x64\x4e'
        b'\x2a\x97\xd9\x4e\x66\xfb\x76\x19\x45\x80\x60\xf7\xbe\x40\xb9'
        b'\xd4\x10\x9f\x84\x65\x56\xdf\x9c\x39\xd8\xe6\x3f\xdb\x7c\x79'
        b'\x31\xe3\xb8\xca\xfc\x79\x9b\x23\xdc\x72\x7c\x4c\x55\x0e\x36'
        b'\x2a\xe0\xeb\xcc\xaa\xa3\x06\x54\xa3\x98\x19\xdc\xa4\x66\x31'
        b'\xd0\x98\x02\x4f\xeb\x32\x16\x61\xec\x97\xca\xce\x92\xa0\x8f'
        b'\x3c\x52\xe8\xdb\x86\x10\x9f\xee\x3f\xa6\xbd\x40\x63\x06\x99'
        b'\x01\xb3\x13\x97\xdc\xe8\x2e\xd1\x10\x8f\xab\x31\x49\xcb\x87'
        b'\x71\x2f\x5e\xf2\x78\xa9\xb4\x3c\x65\xb1\xb2\xd0\x82\xa1\x95'
        b'\x68\x67\x44\xd7\x5e\xec\xb4\x2f\x79\x40\x7e\xd4\xbc\x84\xdb'
        b'\xb9\x8c\xdd\x8d\x9c\x01\x15\xcd\x52\x83\x3f\x06\x67\xfd\xa1'
        b'\x2d\x2b\x07\xba\x32\x62\x21\x07\x2f\x02\x03\x01\x00\x01\x02'
        b'\x82\x01\x00\x30\xe9\x54\x29\xbb\x92\xa6\x28\x29\xf3\x91\x2f'
        b'\xe9\x2a\xaa\x6e\x77\xec\xed\x9c\xbe\x01\xee\x83\x2e\x0f\xd4'
        b'\x62\x06\xd5\x22\xaf\x5f\x44\x00\x5d\xb5\x45\xee\x8c\x57\xc3'
        b'\xe9\x92\x03\x94\x52\x8f\x5b\x9f\x5e\x73\x84\x06\xdf\xf7\xaf'
        b'\x9b\xe7\xb4\x83\xd1\xee\x0c\x41\x3b\x72\xf8\x83\x56\x98\x45'
        b'\x31\x98\x66\xdb\x19\x15\xe4\xcb\x77\xd2\xbc\x61\x3c\x1e\xa9'
        b'\xc5\xa5\x1c\x2f\xec\x3f\x92\x91\xfe\x5c\x38\xcc\x50\x97\x49'
        b'\x07\xc0\x38\x3f\x74\x31\xfb\x17\xc8\x79\x60\x50\x6f\xcc\x1d'
        b'\xfc\x42\xd5\x4a\x07\xd1\x2d\x13\x5e\xa9\x82\xf4\xd0\xa5\xd5'
        b'\xb3\x4e\x3f\x14\xe0\x44\x86\xa4\xa2\xaa\x2f\xe8\x1d\x82\x78'
        b'\x83\x13\x6b\x4a\x82\x0d\x5f\xbd\x4f\x1d\x56\xda\x12\x29\x08'
        b'\xca\x0c\xe2\xe0\x76\x55\xc8\xcb\xad\xdc\xb1\x3a\x71\xe1\xf3'
        b'\x7d\x28\xfb\xd5\xfb\x67\xf9\x48\xb4\x4f\x39\x0b\x39\xbf\x8d'
        b'\xa0\x13\xf7\xd6\x16\x87\x0b\xfb\x1f\x0a\xba\x4a\x83\xb4\x2d'
        b'\x50\xff\x6a\xf5\xd4\x6a\xe9\xd6\x5c\x23\x5e\xea\xe5\xde\xe8'
        b'\x11\xd1\x13\x78\x34\x4a\x85\x3d\xaf\x9b\xb6\xf1\xd9\xb2\xc6'
        b'\x78\x5d\x70\xd8\x7f\x41\xfd\x5f\x35\xba\x98\xe2\x01\xa8\x76'
        b'\x45\x59\xde\x71\x02\x81\x81\x00\xec\x7c\x74\xa3\x47\x58\x1d'
        b'\xf9\x21\xf0\xff\x60\x3d\x49\xa5\xd2\xd6\x4f\x4b\x79\x72\xed'
        b'\xf9\x46\xc3\x41\xd6\xe3\x60\xeb\x21\xe4\xba\x13\xf8\x43\x7f'
        b'\xba\xd3\xbb\xd1\x1c\x83\x62\xa8\xe5\x87\x3a\x89\xcd\xc8\x8a'
        b'\x4e\xe0\x16\xe5\x25\x4f\x0b\xa8\x10\xb8\x2a\x69\x03\x6f\x4a'
        b'\x9e\xda\xbb\xc7\x5f\x8b\xc3\xfe\x30\x1b\xde\x3b\xa6\x85\xdb'
        b'\xeb\x4b\x4b\x76\x0d\xc1\x2b\x99\x81\x15\x33\x91\x93\x90\x13'
        b'\xa8\x0c\x15\xab\xbb\x7e\xd8\xdb\x52\xe5\x2f\xc9\xba\x7c\xec'
        b'\xe7\x1a\xd1\xa2\x50\xc5\x9d\x25\xf8\x2a\x7b\xd5\x97\xa2\x63'
        b'\xdd\x02\x81\x81\x00\xc2\x39\x76\x53\x55\x74\x4f\x10\x58\x67'
        b'\xaa\x7a\x8b\x12\xb6\x5e\xe8\x42\x64\xc9\x2c\x06\xf3\x08\x2d'
        b'\x39\xd0\xa6\xaf\xae\xb4\x6e\x87\x18\xd6\x2f\x6f\x57\xe4\x5a'
        b'\x33\x58\x80\x44\x75\xfa\xbb\xfb\x2e\x32\x19\x33\xfb\x72\x91'
        b'\x8a\x7c\xf1\x20\x6e\x60\x42\xcc\xa2\x5a\x64\xe9\x15\x5d\xbd'
        b'\xf1\x6f\x6f\x91\x1b\x66\xb0\x24\x03\x9f\x69\xb2\xf7\x4c\xaf'
        b'\xe1\xee\xac\x2c\x8d\x27\x83\xb9\x7f\x37\x7a\xfb\x0b\x02\xcb'
        b'\x34\x85\x7f\x0a\xa7\xb2\x68\xde\x34\xb2\xec\xc4\xf0\x08\xe0'
        b'\x12\x06\xb9\x8d\x3b\x9a\xe9\xb3\xf9\x9b\xec\x7c\x7b\x02\x81'
        b'\x81\x00\x9e\xb9\x6d\xc3\xc5\x77\xe4\x2e\x39\xd4\xba\x63\x0a'
        b'\xdf\xaa\x97\xd7\x55\xc3\x6f\x91\x6f\x1e\x37\x9b\x88\x4e\x45'
        b'\xb0\xe0\x40\x90\x77\x40\x3e\x0a\x77\xe9\x9a\x81\x5d\xfa\x08'
        b'\x49\x28\xd9\x5d\xa9\x31\xa2\xd7\xed\xd4\xc0\xdd\x3d\x11\x8c'
        b'\x7b\x63\x63\x4d\x68\xd1\xb1\x07\x7a\x8b\x22\x7e\x94\x73\x91'
        b'\xa8\x8b\xac\x18\x98\x51\x6b\x14\x3f\x26\x2f\x14\x47\xf9\x35'
        b'\x65\x21\x13\x9d\x7a\x4e\x44\x3f\x98\xa1\xda\xf2\x94\xa0\x34'
        b'\xa4\x32\x98\xf1\xd0\xe0\x51\xf5\xd5\x3f\xcc\x25\x56\x0f\x66'
        b'\x83\x72\x5f\x9d\x8c\x1e\x31\x37\x42\x55\x02\x81\x81\x00\xb1'
        b'\xd7\x7d\xe2\x36\x68\x26\x91\x37\xf1\xcc\x67\x22\xfb\x02\x64'
        b'\x8a\xd5\x68\x85\xd0\x3b\x98\xc3\x8e\xed\xd6\x81\x1a\x72\xa5'
        b'\x22\x63\xaf\xb9\x47\x7b\xf3\x85\xd3\x96\x1a\x5e\x70\xd1\x7a'
        b'\xc2\x2f\xf0\x0f\xcd\x86\x0c\xa2\xce\x63\x79\x9e\x2c\xed\x04'
        b'\x55\x86\x1c\xcf\x1a\x81\x56\xa0\x1c\x71\x7b\x71\x33\xf4\x5c'
        b'\x25\xc3\x04\x52\x2e\xad\xc1\xc5\xc5\x72\xe2\x61\x62\xf5\xe9'
        b'\x0d\xb3\x87\xaa\x5c\x80\x8c\x87\x85\x5b\xd5\x35\x0b\xa3\x9c'
        b'\x38\x6b\xe6\xe3\x42\xeb\xdd\x42\xb3\x31\xae\x58\xae\xda\xba'
        b'\x31\x6e\x2b\x8b\xbb\x92\x0b\x02\x81\x81\x00\xdf\x76\xa5\x63'
        b'\x4f\x8b\x97\x98\x6c\x0e\x87\x5c\xf8\x3f\x3b\xfa\x18\x2a\x1c'
        b'\xfb\xa1\xa8\x6d\x78\x38\x0e\xfb\xc2\x52\x33\xfd\x31\x1f\xb6'
        b'\xfb\x9b\x17\xd0\x06\x3f\x7f\xe6\x95\x08\x3d\x39\xfc\xd8\xf4'
        b'\x46\xaa\x40\xc1\x47\x34\xdf\x36\x54\xe5\x9b\x4b\xda\xe3\x5e'
        b'\xe9\x70\xe3\x12\xe8\x1f\x16\xd9\x73\x79\xae\xbe\xad\xb0\xfa'
        b'\x2a\x91\x52\xfa\x7c\x4f\x24\x0f\x18\xc9\x66\x11\xa4\xd8\x69'
        b'\x45\x61\x96\x41\xa9\x07\x79\xda\xf7\x06\xd3\x2d\x1a\xcd\x21'
        b'\xa4\xa3\x40\x40\x6e\xf6\x1c\xa5\xad\x49\xf2\x50\x31\x7b\xe7'
        b'\xd9\x19\x62\x70')
    return key_der


def get_public_key_der():
    """Returns a public key in DER format

    This key was created by issuing the following openssl commands:
    openssl genrsa -out private.pem 2048
    openssl rsa -in private.pem -pubout > public.pem
    The byte string returned by this function is the contents
    of the public.der file.
    """
    key_der = (
        b'\x30\x82\x01\x22\x30\x0d\x06\x09\x2a\x86\x48\x86\xf7\x0d\x01'
        b'\x01\x01\x05\x00\x03\x82\x01\x0f\x00\x30\x82\x01\x0a\x02\x82'
        b'\x01\x01\x00\xb3\x6b\x65\x68\x0d\x79\x81\x50\xc9\xb0\x8c\x5b'
        b'\xbd\x17\xa3\x0c\xe6\xaf\xc0\x67\x55\xa3\x9d\x60\x36\x60\xd7'
        b'\x4d\xcb\x6d\xfb\x4e\xb1\x8d\xfe\x7a\x1b\x0c\x3b\xfc\x14\x10'
        b'\x69\x50\xf9\x87\x35\x9d\x38\x1f\x52\xf2\xc4\x57\x0f\xf1\x17'
        b'\x85\xad\xc2\x17\xa6\x27\xec\x45\xeb\xb6\x94\x05\x9a\xa9\x13'
        b'\xf1\xa2\xfb\xb9\x0a\xe0\x21\x7d\xe7\x0a\xbf\xe4\x61\x8c\xb5'
        b'\x4b\x27\x42\x3e\x31\x92\x1b\xef\x64\x4e\x2a\x97\xd9\x4e\x66'
        b'\xfb\x76\x19\x45\x80\x60\xf7\xbe\x40\xb9\xd4\x10\x9f\x84\x65'
        b'\x56\xdf\x9c\x39\xd8\xe6\x3f\xdb\x7c\x79\x31\xe3\xb8\xca\xfc'
        b'\x79\x9b\x23\xdc\x72\x7c\x4c\x55\x0e\x36\x2a\xe0\xeb\xcc\xaa'
        b'\xa3\x06\x54\xa3\x98\x19\xdc\xa4\x66\x31\xd0\x98\x02\x4f\xeb'
        b'\x32\x16\x61\xec\x97\xca\xce\x92\xa0\x8f\x3c\x52\xe8\xdb\x86'
        b'\x10\x9f\xee\x3f\xa6\xbd\x40\x63\x06\x99\x01\xb3\x13\x97\xdc'
        b'\xe8\x2e\xd1\x10\x8f\xab\x31\x49\xcb\x87\x71\x2f\x5e\xf2\x78'
        b'\xa9\xb4\x3c\x65\xb1\xb2\xd0\x82\xa1\x95\x68\x67\x44\xd7\x5e'
        b'\xec\xb4\x2f\x79\x40\x7e\xd4\xbc\x84\xdb\xb9\x8c\xdd\x8d\x9c'
        b'\x01\x15\xcd\x52\x83\x3f\x06\x67\xfd\xa1\x2d\x2b\x07\xba\x32'
        b'\x62\x21\x07\x2f\x02\x03\x01\x00\x01')
    return key_der


def get_symmetric_key():
    """Returns symmetric key bytes

    16 bytes that were randomly generated. Form a 128 bit key.
    """
    symmetric_key = (
        b'\x92\xcf\x1e\xd9\x54\xea\x30\x70\xd8\xc2\x48\xae\xc1\xc8\x72\xa3')
    return symmetric_key



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Key manager implementation for Vault
"""

import binascii
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import Encoding
from cryptography.hazmat.primitives.serialization import NoEncryption
from cryptography.hazmat.primitives.serialization import PrivateFormat
from cryptography.hazmat.primitives.serialization import PublicFormat

import os
import time
import uuid

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import timeutils
import requests

from castellan.common import exception
from castellan.common.objects import private_key as pri_key
from castellan.common.objects import public_key as pub_key
from castellan.common.objects import symmetric_key as sym_key
from castellan.i18n import _
from castellan.key_manager import key_manager

_DEFAULT_VAULT_URL = "http://127.0.0.1:8200"
_DEFAULT_MOUNTPOINT = "secret"
_DEFAULT_VERSION = 2

_vault_opts = [
    cfg.StrOpt('root_token_id',
               secret=True,
               help='root token for vault'),
    cfg.StrOpt('approle_role_id',
               secret=True,
               help='AppRole role_id for authentication with vault'),
    cfg.StrOpt('approle_secret_id',
               secret=True,
               help='AppRole secret_id for authentication with vault'),
    cfg.StrOpt('kv_mountpoint',
               default=_DEFAULT_MOUNTPOINT,
               help='Mountpoint of KV store in Vault to use'),
    cfg.StrOpt('kv_path',
               help='Path relative to root of KV store in Vault to use.'
               ),
    cfg.IntOpt('kv_version',
               default=_DEFAULT_VERSION,
               choices=(1, 2),
               help='Version of KV store in Vault to use.'),
    cfg.URIOpt('vault_url',
               default=_DEFAULT_VAULT_URL,
               schemes=('http', 'https'),
               help='Use this endpoint to connect to Vault'),
    cfg.StrOpt('ssl_ca_crt_file',
               help='Absolute path to ca cert file'),
    cfg.BoolOpt('use_ssl',
                default=False,
                deprecated_for_removal=True,
                deprecated_reason='This option has no effect.',
                help=_('SSL Enabled/Disabled')),
    cfg.StrOpt("namespace",
               help=_("Vault Namespace to use for all requests to Vault. "
                      "Vault Namespaces feature is available only in "
                      "Vault Enterprise")),
    cfg.FloatOpt('timeout',
                 default=60,
                 help=_('Timeout (in seconds) in each request to Vault')),
]

_VAULT_OPT_GROUP = 'vault'

_EXCEPTIONS_BY_CODE = [
    requests.codes['internal_server_error'],
    requests.codes['service_unavailable'],
    requests.codes['request_timeout'],
    requests.codes['gateway_timeout'],
    requests.codes['precondition_failed'],
]

LOG = logging.getLogger(__name__)


class VaultKeyManager(key_manager.KeyManager):
    """Key Manager Interface that wraps the Vault REST API."""

    def __init__(self, configuration):
        self._conf = configuration
        self._conf.register_opts(_vault_opts, group=_VAULT_OPT_GROUP)
        self._session = requests.Session()
        self._root_token_id = self._conf.vault.root_token_id
        self._approle_role_id = self._conf.vault.approle_role_id
        self._approle_secret_id = self._conf.vault.approle_secret_id
        self._cached_approle_token_id = None
        self._approle_token_ttl = None
        self._approle_token_issue = None
        self._kv_mountpoint = self._conf.vault.kv_mountpoint
        self._kv_path = self._conf.vault.kv_path
        self._kv_version = self._conf.vault.kv_version
        self._vault_url = self._conf.vault.vault_url
        self._namespace = self._conf.vault.namespace
        self._timeout = self._conf.vault.timeout
        if self._vault_url.startswith("https://"):
            self._verify_server = self._conf.vault.ssl_ca_crt_file or True
        else:
            self._verify_server = False

    def _get_url(self):
        if not self._vault_url.endswith('/'):
            self._vault_url += '/'
        return self._vault_url

    def _get_resource_url(self, key_id=None):
        return '{}v1/{}/{}{}{}'.format(
            self._get_url(),
            self._kv_mountpoint,

            '' if self._kv_version == 1 else
            'data/' if key_id else
            'metadata/',  # no key_id is for listing and 'data/' doesn't works
            (self._kv_path + '/') if self._kv_path else '',
            key_id if key_id else '?list=true')

    @property
    def _approle_token_id(self):
        if (all((self._approle_token_issue, self._approle_token_ttl)) and
                timeutils.is_older_than(self._approle_token_issue,
                                        self._approle_token_ttl)):
            self._cached_approle_token_id = None
        return self._cached_approle_token_id

    def _set_namespace(self, headers):
        if self._namespace:
            headers["X-Vault-Namespace"] = self._namespace
        return headers

    def _build_auth_headers(self):
        if self._root_token_id:
            return self._set_namespace(
                {'X-Vault-Token': self._root_token_id})

        if self._approle_token_id:
            return self._set_namespace(
                {'X-Vault-Token': self._approle_token_id})

        if self._approle_role_id:
            params = {
                'role_id': self._approle_role_id
            }
            if self._approle_secret_id:
                params['secret_id'] = self._approle_secret_id
            approle_login_url = '{}v1/auth/approle/login'.format(
                self._get_url()
            )
            token_issue_utc = timeutils.utcnow()
            headers = self._set_namespace({})
            try:
                resp = self._session.post(url=approle_login_url,
                                          json=params,
                                          headers=headers,
                                          verify=self._verify_server,
                                          timeout=self._timeout)
            except Exception as ex:
                raise exception.KeyManagerError(str(ex))

            if resp.status_code in _EXCEPTIONS_BY_CODE:
                raise exception.KeyManagerError(resp.reason)
            if resp.status_code == requests.codes['forbidden']:
                raise exception.Forbidden()

            resp_data = resp.json()

            if resp.status_code == requests.codes['bad_request']:
                raise exception.KeyManagerError(', '.join(resp_data['errors']))

            self._cached_approle_token_id = resp_data['auth']['client_token']
            self._approle_token_issue = token_issue_utc
            self._approle_token_ttl = resp_data['auth']['lease_duration']
            return self._set_namespace(
                {'X-Vault-Token': self._approle_token_id})

        return {}

    def _do_http_request(self, method, resource, json=None):
        headers = self._build_auth_headers()

        try:
            resp = method(resource, headers=headers, json=json,
                          verify=self._verify_server, timeout=self._timeout)
        except Exception as ex:
            raise exception.KeyManagerError(str(ex))

        if resp.status_code in _EXCEPTIONS_BY_CODE:
            raise exception.KeyManagerError(resp.reason)
        if resp.status_code == requests.codes['forbidden']:
            raise exception.Forbidden()

        return resp

    def create_key_pair(self, context, algorithm, length,
                        expiration=None, name=None):
        """Creates an asymmetric key pair."""

        if algorithm.lower() != 'rsa':
            raise NotImplementedError(
                "VaultKeyManager only implements rsa keys"
            )

        priv_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=length,
            backend=default_backend()
        )

        private_key = pri_key.PrivateKey(
            'RSA',
            length,
            priv_key.private_bytes(
                Encoding.PEM, PrivateFormat.PKCS8, NoEncryption()
            )
        )

        private_key_id = uuid.uuid4().hex
        private_id = self._store_key_value(
            private_key_id,
            private_key
        )

        # pub_key = priv_key.public_key()
        public_key = pub_key.PublicKey(
            'RSA',
            length,
            priv_key.public_key().public_bytes(
                Encoding.PEM, PublicFormat.SubjectPublicKeyInfo
            )
        )

        public_key_id = uuid.uuid4().hex
        public_id = self._store_key_value(
            public_key_id,
            public_key
        )

        return private_id, public_id

    def _store_key_value(self, key_id, value):

        type_value = self._secret_type_dict.get(type(value))
        if type_value is None:
            raise exception.KeyManagerError(
                "Unknown type for value : %r" % value)

        record = {
            'type': type_value,
            'value': binascii.hexlify(value.get_encoded()).decode('utf-8'),
            'algorithm': (value.algorithm if hasattr(value, 'algorithm')
                          else None),
            'bit_length': (value.bit_length if hasattr(value, 'bit_length')
                           else None),
            'name': value.name,
            'created': value.created
        }
        if self._kv_version > 1:
            record = {'data': record}

        self._do_http_request(self._session.post,
                              self._get_resource_url(key_id),
                              json=record)

        return key_id

    def create_key(self, context, algorithm, length, name=None, **kwargs):
        """Creates a symmetric key."""

        if length % 8:
            msg = _("Length must be multiple of 8.")
            raise ValueError(msg)

        key_id = uuid.uuid4().hex
        key_value = os.urandom((length or 256) // 8)
        key = sym_key.SymmetricKey(algorithm,
                                   length or 256,
                                   key_value,
                                   key_id,
                                   name or int(time.time()))

        return self._store_key_value(key_id, key)

    def store(self, context, key_value, **kwargs):
        """Stores (i.e., registers) a key with the key manager."""

        key_id = uuid.uuid4().hex
        return self._store_key_value(key_id, key_value)

    def get(self, context, key_id, metadata_only=False):
        """Retrieves the key identified by the specified id."""

        if not key_id:
            raise exception.KeyManagerError('key identifier not provided')

        resp = self._do_http_request(self._session.get,
                                     self._get_resource_url(key_id))

        if resp.status_code == requests.codes['not_found']:
            raise exception.ManagedObjectNotFoundError(uuid=key_id)

        record = resp.json()['data']
        if self._kv_version > 1:
            record = record['data']

        key = None if metadata_only else binascii.unhexlify(record['value'])

        clazz = None
        for type_clazz, type_name in self._secret_type_dict.items():
            if type_name == record['type']:
                clazz = type_clazz

        if clazz is None:
            raise exception.KeyManagerError(
                "Unknown type : %r" % record['type'])

        if hasattr(clazz, 'algorithm') and hasattr(clazz, 'bit_length'):
            return clazz(record['algorithm'],
                         record['bit_length'],
                         key,
                         record['name'],
                         record['created'],
                         key_id)
        else:
            return clazz(key,
                         record['name'],
                         record['created'],
                         key_id)

    def delete(self, context, key_id, force=False):
        """Represents deleting the key.

        The 'force' parameter is not used whatsoever and only kept to allow
        consistency with the Barbican implementation.
        """

        if not key_id:
            raise exception.KeyManagerError('key identifier not provided')

        resp = self._do_http_request(self._session.delete,
                                     self._get_resource_url(key_id))

        if resp.status_code == requests.codes['not_found']:
            raise exception.ManagedObjectNotFoundError(uuid=key_id)

    def add_consumer(self, context, managed_object_id, consumer_data):
        raise NotImplementedError(
            "VaultKeyManager does not implement adding consumers"
        )

    def remove_consumer(self, context, managed_object_id, consumer_data):
        raise NotImplementedError(
            "VaultKeyManager does not implement deleting consumers"
        )

    def list(self, context, object_type=None, metadata_only=False):
        """Lists the managed objects given the criteria."""

        if object_type and object_type not in self._secret_type_dict:
            msg = _("Invalid secret type: %s") % object_type
            raise exception.KeyManagerError(reason=msg)

        resp = self._do_http_request(self._session.get,
                                     self._get_resource_url())

        if resp.status_code == requests.codes['not_found']:
            keys = []
        else:
            keys = resp.json()['data']['keys']

        objects = []
        for obj_id in keys:
            try:
                obj = self.get(context, obj_id, metadata_only=metadata_only)
                if object_type is None or isinstance(obj, object_type):
                    objects.append(obj)
            except exception.ManagedObjectNotFoundError as e:
                LOG.warning("Error occurred while retrieving object "
                            "metadata, not adding it to the list: %s", e)
                pass
        return objects

    def list_options_for_discovery(self):
        return [(_VAULT_OPT_GROUP, _vault_opts)]



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for the mock key manager.
"""

from cryptography.hazmat import backends
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from oslo_context import context

from castellan.common import exception
from castellan.common.objects import symmetric_key as sym_key
from castellan.tests.unit.key_manager import mock_key_manager as mock_key_mgr
from castellan.tests.unit.key_manager import test_key_manager as test_key_mgr


def get_cryptography_private_key(private_key):
    crypto_private_key = serialization.load_der_private_key(
        bytes(private_key.get_encoded()),
        password=None,
        backend=backends.default_backend())
    return crypto_private_key


def get_cryptography_public_key(public_key):
    crypto_public_key = serialization.load_der_public_key(
        bytes(public_key.get_encoded()),
        backend=backends.default_backend())
    return crypto_public_key


class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):

    def _create_key_manager(self):
        return mock_key_mgr.MockKeyManager()

    def setUp(self):
        super(MockKeyManagerTestCase, self).setUp()

        self.context = context.RequestContext('fake', 'fake')

    def cleanUp(self):
        super(MockKeyManagerTestCase, self).cleanUp()

        self.key_mgr.keys = {}

    def test_create_key(self):
        key_id_1 = self.key_mgr.create_key(self.context)
        key_id_2 = self.key_mgr.create_key(self.context)
        # ensure that the UUIDs are unique
        self.assertNotEqual(key_id_1, key_id_2)

    def test_create_key_with_length(self):
        for length in [64, 128, 256]:
            key_id = self.key_mgr.create_key(self.context, length=length)
            key = self.key_mgr.get(self.context, key_id)
            self.assertEqual(length / 8, len(key.get_encoded()))
            self.assertIsNotNone(key.id)

    def test_create_key_with_name(self):
        name = 'my key'
        key_id = self.key_mgr.create_key(self.context, name=name)
        key = self.key_mgr.get(self.context, key_id)
        self.assertEqual(name, key.name)
        self.assertIsNotNone(key.id)

    def test_create_key_with_algorithm(self):
        algorithm = 'DES'
        key_id = self.key_mgr.create_key(self.context, algorithm=algorithm)
        key = self.key_mgr.get(self.context, key_id)
        self.assertEqual(algorithm, key.algorithm)
        self.assertIsNotNone(key.id)

    def test_create_key_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.create_key, None)

    def test_create_key_pair(self):
        for length in [2048, 3072, 4096]:
            name = str(length) + ' key'
            private_key_uuid, public_key_uuid = self.key_mgr.create_key_pair(
                self.context, 'RSA', length, name=name)

            private_key = self.key_mgr.get(self.context, private_key_uuid)
            self.assertIsNotNone(private_key.id)
            public_key = self.key_mgr.get(self.context, public_key_uuid)
            self.assertIsNotNone(public_key.id)

            crypto_private_key = get_cryptography_private_key(private_key)
            crypto_public_key = get_cryptography_public_key(public_key)

            self.assertEqual(name, private_key.name)
            self.assertEqual(name, public_key.name)

            self.assertEqual(length, crypto_private_key.key_size)
            self.assertEqual(length, crypto_public_key.key_size)

    def test_create_key_pair_encryption(self):
        private_key_uuid, public_key_uuid = self.key_mgr.create_key_pair(
            self.context, 'RSA', 2048)

        private_key = self.key_mgr.get(self.context, private_key_uuid)
        public_key = self.key_mgr.get(self.context, public_key_uuid)

        crypto_private_key = get_cryptography_private_key(private_key)
        crypto_public_key = get_cryptography_public_key(public_key)

        message = b'secret plaintext'
        ciphertext = crypto_public_key.encrypt(
            message,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA1()),
                algorithm=hashes.SHA1(),
                label=None))
        plaintext = crypto_private_key.decrypt(
            ciphertext,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA1()),
                algorithm=hashes.SHA1(),
                label=None))

        self.assertEqual(message, plaintext)

    def test_create_key_pair_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.create_key_pair, None, 'RSA', 2048)

    def test_create_key_pair_invalid_algorithm(self):
        self.assertRaises(ValueError,
                          self.key_mgr.create_key_pair,
                          self.context, 'DSA', 2048)

    def test_create_key_pair_invalid_length(self):
        self.assertRaises(ValueError,
                          self.key_mgr.create_key_pair,
                          self.context, 'RSA', 10)

    def test_store_and_get_key(self):
        secret_key = bytes(b'0' * 64)
        _key = sym_key.SymmetricKey('AES', 64 * 8, secret_key)
        key_id = self.key_mgr.store(self.context, _key)

        actual_key = self.key_mgr.get(self.context, key_id)
        self.assertEqual(_key, actual_key)

        self.assertIsNotNone(actual_key.id)

    def test_store_key_and_get_metadata(self):
        secret_key = bytes(b'0' * 64)
        _key = sym_key.SymmetricKey('AES', 64 * 8, secret_key)
        key_id = self.key_mgr.store(self.context, _key)

        actual_key = self.key_mgr.get(self.context,
                                      key_id,
                                      metadata_only=True)
        self.assertIsNone(actual_key.get_encoded())
        self.assertTrue(actual_key.is_metadata_only())

        self.assertIsNotNone(actual_key.id)

    def test_store_key_and_get_metadata_and_get_key(self):
        secret_key = bytes(b'0' * 64)
        _key = sym_key.SymmetricKey('AES', 64 * 8, secret_key)
        key_id = self.key_mgr.store(self.context, _key)

        actual_key = self.key_mgr.get(self.context,
                                      key_id,
                                      metadata_only=True)
        self.assertIsNone(actual_key.get_encoded())
        self.assertTrue(actual_key.is_metadata_only())

        actual_key = self.key_mgr.get(self.context,
                                      key_id,
                                      metadata_only=False)
        self.assertIsNotNone(actual_key.get_encoded())
        self.assertFalse(actual_key.is_metadata_only())

        self.assertIsNotNone(actual_key.id)

    def test_store_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.store, None, None)

    def test_get_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.get, None, None)

    def test_get_unknown_key(self):
        self.assertRaises(
            exception.ManagedObjectNotFoundError, self.key_mgr.get,
            self.context, None)

    def test_delete_key(self):
        key_id = self.key_mgr.create_key(self.context)
        self.key_mgr.delete(self.context, key_id)

        self.assertRaises(
            exception.ManagedObjectNotFoundError, self.key_mgr.get,
            self.context, key_id)

    def test_delete_null_context(self):
        self.assertRaises(exception.Forbidden,
                          self.key_mgr.delete, None, None)

    def test_delete_unknown_key(self):
        self.assertRaises(
            exception.ManagedObjectNotFoundError, self.key_mgr.delete,
            self.context, None)

    def test_list_null_context(self):
        self.assertRaises(exception.Forbidden, self.key_mgr.list, None)

    def test_list_keys(self):
        key1 = sym_key.SymmetricKey('AES', 64 * 8, bytes(b'0' * 64))
        self.key_mgr.store(self.context, key1)
        key2 = sym_key.SymmetricKey('AES', 32 * 8, bytes(b'0' * 32))
        self.key_mgr.store(self.context, key2)

        keys = self.key_mgr.list(self.context)
        self.assertEqual(2, len(keys))
        self.assertIn(key1, keys)
        self.assertIn(key2, keys)

        for key in keys:
            self.assertIsNotNone(key.id)

    def test_list_keys_metadata_only(self):
        key1 = sym_key.SymmetricKey('AES', 64 * 8, bytes(b'0' * 64))
        self.key_mgr.store(self.context, key1)
        key2 = sym_key.SymmetricKey('AES', 32 * 8, bytes(b'0' * 32))
        self.key_mgr.store(self.context, key2)

        keys = self.key_mgr.list(self.context, metadata_only=True)
        self.assertEqual(2, len(keys))
        bit_length_list = [key1.bit_length, key2.bit_length]
        for key in keys:
            self.assertTrue(key.is_metadata_only())
            self.assertIn(key.bit_length, bit_length_list)

        for key in keys:
            self.assertIsNotNone(key.id)



# Copyright (c) 2015 IBM
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Keystone Token Credential

This module defines the Keystone Token credential.
"""
from castellan.common.credentials import token


class KeystoneToken(token.Token):
    """This class represents a keystone token credential."""

    def __init__(self, token, auth_url=None, trust_id=None, domain_id=None,
                 domain_name=None, project_id=None, project_name=None,
                 project_domain_id=None, project_domain_name=None,
                 reauthenticate=True):
        """Create a new Keystone Token Credential.

        :param string token: Token for authentication. The type of token
        formats accepted are UUID, PKI, and Fernet.
        :param string auth_url: Use this endpoint to connect to Keystone.
        :param string trust_id: Trust ID for trust scoping.
        :param string domain_id: Domain ID for domain scoping.
        :param string domain_name: Domain name for domain scoping.
        :param string project_id: Project ID for project scoping.
        :param string project_name: Project name for project scoping.
        :param string project_domain_id: Project's domain ID for project.
        :param string project_domain_name: Project's domain name for project.
        :param bool reauthenticate: Allow fetching a new token if the current
        one is going to expire. (optional) default True
        """

        self._auth_url = auth_url
        self._trust_id = trust_id
        self._domain_id = domain_id
        self._domain_name = domain_name
        self._project_id = project_id
        self._project_name = project_name
        self._project_domain_id = project_domain_id
        self._project_domain_name = project_domain_name
        self._reauthenticate = reauthenticate

        super(KeystoneToken, self).__init__(token)

    @property
    def auth_url(self):
        """This method returns an auth_url."""
        return self._auth_url

    @property
    def trust_id(self):
        """This method returns a trust_id."""
        return self._trust_id

    @property
    def domain_id(self):
        """This method returns a domain_id."""
        return self._domain_id

    @property
    def domain_name(self):
        """This method returns a domain_name."""
        return self._domain_name

    @property
    def project_id(self):
        """This method returns a project_id."""
        return self._project_id

    @property
    def project_name(self):
        """This method returns a project_name."""
        return self._project_name

    @property
    def project_domain_id(self):
        """This method returns a project_domain_id."""
        return self._project_domain_id

    @property
    def project_domain_name(self):
        """This method returns a project_domain_name."""
        return self._project_domain_name

    @property
    def reauthenticate(self):
        """This method returns reauthenticate."""
        return self._reauthenticate

    def __eq__(self, other):
        if isinstance(other, KeystoneToken):
            return (
                self._token == other._token and
                self._trust_id == other._trust_id and
                self._domain_id == other._domain_id and
                self._domain_name == other._domain_name and
                self._project_id == other._project_id and
                self._project_name == other._project_name and
                self._project_domain_id == other._project_domain_id and
                self._project_domain_name == other._project_domain_name and
                self._reauthenticate == other._reauthenticate)
        else:
            return False

    def __ne__(self, other):
        result = self.__eq__(other)
        return not result



# Copyright (c) 2021 Mirantis Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for Vault key manager.
"""
import requests_mock

from castellan.key_manager import vault_key_manager
from castellan.tests.unit.key_manager import test_key_manager


class VaultKeyManagerTestCase(test_key_manager.KeyManagerTestCase):

    def _create_key_manager(self):
        return vault_key_manager.VaultKeyManager(self.conf)

    def test_auth_headers_root_token(self):
        self.key_mgr._root_token_id = "spam"
        expected_headers = {"X-Vault-Token": "spam"}
        self.assertEqual(expected_headers,
                         self.key_mgr._build_auth_headers())

    def test_auth_headers_root_token_with_namespace(self):
        self.key_mgr._root_token_id = "spam"
        self.key_mgr._namespace = "ham"
        expected_headers = {"X-Vault-Token": "spam",
                            "X-Vault-Namespace": "ham"}
        self.assertEqual(expected_headers,
                         self.key_mgr._build_auth_headers())

    @requests_mock.Mocker()
    def test_auth_headers_app_role(self, m):
        self.key_mgr._approle_role_id = "spam"
        self.key_mgr._approle_secret_id = "secret"
        m.post(
            "http://127.0.0.1:8200/v1/auth/approle/login",
            json={"auth": {"client_token": "token", "lease_duration": 3600}}
        )
        expected_headers = {"X-Vault-Token": "token"}
        self.assertEqual(expected_headers, self.key_mgr._build_auth_headers())

    @requests_mock.Mocker()
    def test_auth_headers_app_role_with_namespace(self, m):
        self.key_mgr._approle_role_id = "spam"
        self.key_mgr._approle_secret_id = "secret"
        self.key_mgr._namespace = "ham"
        m.post(
            "http://127.0.0.1:8200/v1/auth/approle/login",
            json={"auth": {"client_token": "token", "lease_duration": 3600}}
        )
        expected_headers = {"X-Vault-Token": "token",
                            "X-Vault-Namespace": "ham"}
        self.assertEqual(expected_headers, self.key_mgr._build_auth_headers())



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base Passphrase Class

This module defines the Passphrase class.
"""

from castellan.common.objects import managed_object


class Passphrase(managed_object.ManagedObject):
    """This class represents a passphrase."""

    def __init__(self, passphrase, name=None, created=None, id=None,
                 consumers=[]):
        """Create a new Passphrase object.

        The expected type for the passphrase is a bytestring.
        """
        self._passphrase = passphrase
        super().__init__(name=name, created=created, id=id,
                         consumers=consumers)

    @classmethod
    def managed_type(cls):
        return "passphrase"

    @property
    def format(self):
        return "RAW"

    def get_encoded(self):
        return self._passphrase

    def __eq__(self, other):
        if isinstance(other, Passphrase):
            return (self._passphrase == other._passphrase)
        else:
            return False

    def __ne__(self, other):
        result = self.__eq__(other)
        return not result



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
A mock implementation of a key manager that stores keys in a dictionary.

This key manager implementation is primarily intended for testing. In
particular, it does not store keys persistently. Lack of a centralized key
store also makes this implementation unsuitable for use among different
services.

Note: Instantiating this class multiple times will create separate key stores.
Keys created in one instance will not be accessible from other instances of
this class.
"""

import binascii
import copy
import random

from cryptography.hazmat import backends
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from oslo_utils import uuidutils

from castellan.common import exception
from castellan.common.objects import private_key as pri_key
from castellan.common.objects import public_key as pub_key
from castellan.common.objects import symmetric_key as sym_key
from castellan.key_manager import key_manager


class MockKeyManager(key_manager.KeyManager):
    """Mocking manager for integration tests.

    This mock key manager implementation supports all the methods specified
    by the key manager interface. This implementation stores keys within a
    dictionary, and as a result, it is not acceptable for use across different
    services. Side effects (e.g., raising exceptions) for each method are
    handled as specified by the key manager interface.

    This key manager is not suitable for use in production deployments.
    """

    def __init__(self, configuration=None):
        self.conf = configuration
        self.keys = {}

    def _generate_hex_key(self, key_length):
        # hex digit => 4 bits
        length = int(key_length / 4)
        hex_encoded = self._generate_password(length=length,
                                              symbolgroups='0123456789ABCDEF')
        return hex_encoded

    def _generate_key(self, **kwargs):
        name = kwargs.get('name', None)
        algorithm = kwargs.get('algorithm', 'AES')
        key_length = kwargs.get('length', 256)
        _hex = self._generate_hex_key(key_length)
        return sym_key.SymmetricKey(
            algorithm,
            key_length,
            bytes(binascii.unhexlify(_hex)),
            name)

    def create_key(self, context, **kwargs):
        """Creates a symmetric key.

        This implementation returns a UUID for the created key. The algorithm
        for the key will always be AES. A Forbidden exception is raised if the
        specified context is None.
        """
        if context is None:
            raise exception.Forbidden()

        key = self._generate_key(**kwargs)
        return self.store(context, key)

    def _generate_public_and_private_key(self, length, name):
        crypto_private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=length,
            backend=backends.default_backend())

        private_der = crypto_private_key.private_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption())

        crypto_public_key = crypto_private_key.public_key()

        public_der = crypto_public_key.public_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PublicFormat.SubjectPublicKeyInfo)

        private_key = pri_key.PrivateKey(
            algorithm='RSA',
            bit_length=length,
            key=bytearray(private_der),
            name=name)

        public_key = pub_key.PublicKey(
            algorithm='RSA',
            bit_length=length,
            key=bytearray(public_der),
            name=name)

        return private_key, public_key

    def create_key_pair(self, context, algorithm, length,
                        expiration=None, name=None):
        """Creates an asymmetric key pair.

        This implementation returns UUIDs for the created keys in the order:
            (private, public)
        Forbidden is raised if the context is None.
        """
        if context is None:
            raise exception.Forbidden()

        if algorithm.lower() != 'rsa':
            msg = 'Invalid algorithm: {}, only RSA supported'.format(algorithm)
            raise ValueError(msg)

        valid_lengths = [2048, 3072, 4096]

        if length not in valid_lengths:
            msg = 'Invalid bit length: {}, only {} supported'.format(
                length, valid_lengths)
            raise ValueError(msg)

        private_key, public_key = self._generate_public_and_private_key(length,
                                                                        name)

        private_key_uuid = self.store(context, private_key)
        public_key_uuid = self.store(context, public_key)

        return private_key_uuid, public_key_uuid

    def _generate_key_id(self):
        key_id = uuidutils.generate_uuid()
        while key_id in self.keys:
            key_id = uuidutils.generate_uuid()

        return key_id

    def store(self, context, managed_object, **kwargs):
        """Stores (i.e., registers) a key with the key manager."""
        if context is None:
            raise exception.Forbidden()

        key_id = self._generate_key_id()
        managed_object._id = key_id
        self.keys[key_id] = managed_object

        return key_id

    def get(self, context, managed_object_id, metadata_only=False, **kwargs):
        """Retrieves the key identified by the specified id.

        This implementation returns the key that is associated with the
        specified UUID. A Forbidden exception is raised if the specified
        context is None; a KeyError is raised if the UUID is invalid.
        """
        if context is None:
            raise exception.Forbidden()

        try:
            obj = copy.deepcopy(self.keys[managed_object_id])
        except KeyError:
            raise exception.ManagedObjectNotFoundError(managed_object_id)

        if metadata_only:
            if hasattr(obj, "_key"):
                obj._key = None
            if hasattr(obj, "_data"):
                obj._data = None
            if hasattr(obj, "_passphrase"):
                obj._passphrase = None
        return obj

    def delete(self, context, managed_object_id, **kwargs):
        """Deletes the object identified by the specified id.

        A Forbidden exception is raised if the context is None and a
        KeyError is raised if the UUID is invalid.
        """
        if context is None:
            raise exception.Forbidden()

        try:
            del self.keys[managed_object_id]
        except KeyError:
            raise exception.ManagedObjectNotFoundError(managed_object_id)

    def add_consumer(self, context, managed_object_id, consumer_data):
        if context is None:
            raise exception.Forbidden()
        if managed_object_id not in self.keys:
            raise exception.ManagedObjectNotFoundError(managed_object_id)
        self.keys[managed_object_id].consumers.append(consumer_data)

    def remove_consumer(self, context, managed_object_id, consumer_data):
        if context is None:
            raise exception.Forbidden()
        if managed_object_id not in self.keys:
            raise exception.ManagedObjectNotFoundError(managed_object_id)
        self.keys[managed_object_id].consumers = [
            c for c in self.keys[managed_object_id].consumers
            if c != consumer_data]

    def _generate_password(self, length, symbolgroups):
        """Generate a random password from the supplied symbol groups.

        At least one symbol from each group will be included. Unpredictable
        results if length is less than the number of symbol groups.

        Believed to be reasonably secure (with a reasonable password length!)
        """
        # NOTE(jerdfelt): Some password policies require at least one character
        # from each group of symbols, so start off with one random character
        # from each symbol group
        password = [random.choice(s) for s in symbolgroups]
        # If length < len(symbolgroups), the leading characters will only
        # be from the first length groups. Try our best to not be predictable
        # by shuffling and then truncating.
        random.shuffle(password)
        password = password[:length]
        length -= len(password)

        # then fill with random characters from all symbol groups
        symbols = ''.join(symbolgroups)
        password.extend([random.choice(symbols) for _i in range(length)])

        # Finally, shuffle to ensure first x characters aren't from a
        # predictable group
        random.shuffle(password)

        return ''.join(password)

    def list(self, context, object_type=None, metadata_only=False):
        """Retrieves a list of managed objects that match the criteria.

        A Forbidden exception is raised if the context is None.
        If no search criteria is given, all objects are returned.
        """
        if context is None:
            raise exception.Forbidden()

        objects = []
        for obj_id in self.keys:
            obj = self.get(context, obj_id, metadata_only=metadata_only)
            if object_type is None or isinstance(obj, object_type):
                objects.append(obj)
        return objects



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for Managed Objects.
"""
from castellan.common import exception
from castellan.common import objects
from castellan.tests import base


class ManagedObjectFromDictTestCase(base.TestCase):
    def test_invalid_dict(self):
        self.assertRaises(
            exception.InvalidManagedObjectDictError,
            objects.from_dict,
            {},
        )

    def test_unknown_type(self):
        self.assertRaises(
            exception.UnknownManagedObjectTypeError,
            objects.from_dict,
            {"type": "non-existing-managed-object-type"},
        )
